/*
 * extract.c: extracts temperature and humidity data from the pulse train
 *            read from the DHT11 humidity and temperature sensor
 *
 * copyright U. Raich Dec 2016
 * This program is part of the workshop on embedded systems held at 
 * the University of Cape Coast in the winter semester 2017
 * It has been released under the GNU General Public License
 * see gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

#define LOW  0
#define HIGH 1

int readBit(int **data) {
  int retVal,*dataPtr=*data;
  int counter;
  /* wait for the signal to go high again */
  counter = 0;
  while(!(*dataPtr++)) {
    counter++;
    if (counter > 255)
      return (-1);
  }
    /* 
     now look 3 samples further (30 us) on if the pulse is still high
     in which case we have a 1
     else we have a 0
  */
  if (*(dataPtr+5) == HIGH)
    retVal = HIGH;
  else
    retVal =  LOW;
  counter = 0;
  while(*dataPtr++) {  /* wait until the signal goes low and the high again */
    counter++;
    if (counter > 255)
      return (-1);
  }
  counter = 0;
  while(!(*dataPtr++)) {/* now we are ready to read the next bit */
    counter++;
    if (counter >255)
      return (-1);
  }
  *data = dataPtr; 
  return retVal;
}

int main(int argc,char **argv) {
  int protocolData[2000],*writePtr, *readPtr,tmp;
  int finalData[5],nextBit;
  char *dummy=NULL;
  size_t lineLength = 100;
  ssize_t charsRead;
  int i,j,sum;
    
  /* read the comment text line and forget it */

  charsRead = getline(&dummy,&lineLength,stdin);
  printf("Comment line: %s\n",dummy);
  readPtr = protocolData;
  writePtr = protocolData;
  while(scanf("%d", readPtr++) != EOF) ;
  /*
  while(writePtr != readPtr)
    printf("%1x\n",*writePtr++);
  */
  /* Now we have to interpret the pulse train */

  readPtr = protocolData;
  /* 
   *  the protocol header is 80us low (of which only part is seen because
   *  we call the library ro switch the gpio pin from output to input)
   *  followed by 80 us high
   */ 
  while (!(*readPtr++)) ; /* wait until the pulse is high */ 
  while (*readPtr++) ;    /* wait until it goes high again */
  
  for (i=0; i<5;i++) {
    tmp = 0;
    for (j=0;j<8;j++) {
      tmp = tmp << 1;
      //     printf("data pointer before call: %lx\n",(long)readPtr);
      if ((nextBit=readBit(&readPtr)) == -1) {
	  printf("Protocol error!\n");
	  exit(-1);
	}
	else
	  tmp |= nextBit;
      //      printf("data pointer after call: %lx\n",(long)readPtr);
    }
    finalData[i]=tmp;
  }
  sum = 0;
  for (i=0;i<4;i++) {
    printf("Data extracted: %x\n",finalData[i]);
    sum += finalData[i];
  }
  printf("Checksum calculated: 0x%x, checksum read: 0x%x\n",sum,finalData[4]); 
  printf("Temperature: %d.%d\n",finalData[0],finalData[1]);
  printf("Humidity: %d.%d\n",finalData[2],finalData[3]);
  if (sum != finalData[3])
    exit(-1);
  else
    exit(1);
}
